#include <GL/gl.h>
#include <GL/glu.h>
#include <GL/freeglut.h>
#include <math.h>
#include "opengl-magic.h"

#define phi0 0.5    /* geg. Anfangswert */
#define omega0 0.0  /* geg. Anfangswert */
#define dt 0.05
#define g 9.81
#define l 1.0

double t = 0.0;
double phi_with_sin = phi0;
double omega_with_sin= omega0;
double phi_without_sin = phi0;
double omega_without_sin= omega0;

void draw_pendulum (double phi)
{
  glPushMatrix ();
  glRotatef (90, 1.0, 0.0, 0.0);
  glTranslatef (0.0, 0.0, -0.75);
  glRotatef (180.0 + phi / M_PI * 180.0, 0.0, 1.0, 0.0);
  glTranslatef (0.0, 0.0, -1.0);
  glutSolidCylinder (0.01, 1.0, 13, 1);
  glutSolidSphere (0.1, 31, 10);
  glPopMatrix ();
  glutSwapBuffers ();
}

void draw (void)
{
  glClear (GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
  set_material_color (0.8, 0.8, 1.0);
  draw_pendulum (phi_with_sin);
  set_material_color (1.0, 0.8, 0.8);
  draw_pendulum (phi_without_sin);
  set_material_color (0.8, 1.0, 0.8);
  double A = phi0;
  double B = 0.5 * M_PI;  /* 90° */
  double phi_analytic = A * sin (sqrt (g / l) * t + B);
  draw_pendulum (phi_analytic);
  glFlush ();
}

void timer_handler (int value)
{
  t += dt;
  phi_with_sin += dt * omega_with_sin;                   /* geg. Differentialgleichung */
  omega_with_sin += - dt * g / l * sin (phi_with_sin);   /* geg. Differentialgleichung */
                                                         /* _ohne_ Kleinwinkelnäherung */
  phi_without_sin += dt * omega_without_sin;             /* geg. Differentialgleichung */
  omega_without_sin += - dt * g / l * phi_without_sin;   /* geg. Differentialgleichung */
                                                         /* _mit_ Kleinwinkelnäherung  */
  glutPostRedisplay ();
  glutTimerFunc (50, timer_handler, 0);
}

int main (int argc, char **argv)
{
  init_opengl (&argc, argv, "Pendulum");
  glutDisplayFunc (draw);
  glutTimerFunc (50, timer_handler, 0);
  glutPostRedisplay ();
  glutMainLoop ();
  return 0;
}
